@@ -1,8 +1,10 @@
 # -*- coding: utf-8 -*-
 import six
 import traceback
+from django.utils.module_loading import import_string
 from django import VERSION as DJANGO_VERSION
 
+from django_extensions.collision_resolvers import CollisionResolvingRunner
 
 SHELL_PLUS_DJANGO_IMPORTS = {
     'django.core.cache': ['cache'],
@@ -107,87 +109,42 @@ def import_items(import_directives, style, quiet_load=False):
 def import_objects(options, style):
     from django.apps import apps
     from django import setup
+
     if not apps.ready:
         setup()
 
-    def get_apps_and_models():
-        for app in apps.get_app_configs():
-            if app.models_module:
-                yield app.models_module, app.get_models()
-
-    mongoengine = False
-    try:
-        from mongoengine.base import _document_registry
-        mongoengine = True
-    except:
-        pass
-
     from django.conf import settings
-    imported_objects = {}
 
     dont_load_cli = options.get('dont_load', [])
     dont_load_conf = getattr(settings, 'SHELL_PLUS_DONT_LOAD', [])
     dont_load = dont_load_cli + dont_load_conf
+    dont_load_any_models = '*' in dont_load
     quiet_load = options.get('quiet_load')
-
     model_aliases = getattr(settings, 'SHELL_PLUS_MODEL_ALIASES', {})
     app_prefixes = getattr(settings, 'SHELL_PLUS_APP_PREFIXES', {})
-
-    # Perform pre-imports before any other imports
     SHELL_PLUS_PRE_IMPORTS = getattr(settings, 'SHELL_PLUS_PRE_IMPORTS', {})
-    if SHELL_PLUS_PRE_IMPORTS:
-        if not quiet_load:
-            print(style.SQL_TABLE("# Shell Plus User Imports"))
-        imports = import_items(SHELL_PLUS_PRE_IMPORTS, style, quiet_load=quiet_load)
-        for k, v in six.iteritems(imports):
-            imported_objects[k] = v
 
-    load_models = {}
+    imported_objects = {}
 
-    if mongoengine:
-        for name, mod in six.iteritems(_document_registry):
-            name = name.split('.')[-1]
-            app_name = mod.__module__.split('.')[-2]
-            if app_name in dont_load or ("%s.%s" % (app_name, name)) in dont_load:
-                continue
-
-            load_models.setdefault(mod.__module__, [])
-            load_models[mod.__module__].append(name)
-
-    for app_mod, app_models in get_apps_and_models():
-        if not app_models:
-            continue
-
-        app_name = app_mod.__name__.split('.')[-2]
-        if app_name in dont_load:
-            continue
-
-        for mod in app_models:
-            if "%s.%s" % (app_name, mod.__name__) in dont_load:
-                continue
-
-            if mod.__module__:
-                # Only add the module to the dict if `__module__` is not empty.
-                load_models.setdefault(mod.__module__, [])
-                load_models[mod.__module__].append(mod.__name__)
-
-    if not quiet_load:
-        print(style.SQL_TABLE("# Shell Plus Model Imports"))
-
-    for app_mod, models in sorted(six.iteritems(load_models)):
-        try:
-            app_name = app_mod.split('.')[-2]
-        except IndexError:
-            # Some weird model naming scheme like in Sentry.
-            app_name = app_mod
-        app_aliases = model_aliases.get(app_name, {})
-        prefix = app_prefixes.get(app_name)
-        model_labels = []
-
-        for model_name in sorted(models):
+    def get_dict_from_names_to_possible_models():
+        """
+        Collects dictionary from names to possible models. Model is represented as his full path.
+        Name of model can be alias if SHELL_PLUS_MODEL_ALIASES or SHELL_PLUS_APP_PREFIXES is specified for this model.
+        This dictionary is used by collision resolver.
+        At this phase we can't import any models, because collision resolver can change results.
+        :return: Dict[str, List[str]]. Key is name, value is list of full model's path's.
+        """
+        models_to_import = {}
+        for app_mod, models in sorted(six.iteritems(load_models)):
             try:
-                imported_object = getattr(__import__(app_mod, {}, {}, [model_name]), model_name)
+                app_name = app_mod.split('.')[-2]
+            except IndexError:
+                # Some weird model naming scheme like in Sentry.
+                app_name = app_mod
+            app_aliases = model_aliases.get(app_name, {})
+            prefix = app_prefixes.get(app_name)
 
+            for model_name in sorted(models):
                 if "%s.%s" % (app_name, model_name) in dont_load:
                     continue
 
@@ -199,21 +156,88 @@ def import_objects(options, style):
                     else:
                         alias = model_name
 
-                imported_objects[alias] = imported_object
-                if model_name == alias:
-                    model_labels.append(model_name)
-                else:
-                    model_labels.append("%s (as %s)" % (model_name, alias))
+                models_to_import.setdefault(alias, [])
+                models_to_import[alias].append("%s.%s" % (app_mod, model_name))
+        return models_to_import
 
-            except AttributeError as e:
-                if options.get("traceback"):
-                    traceback.print_exc()
-                if not quiet_load:
-                    print(style.ERROR("Failed to import '%s' from '%s' reason: %s" % (model_name, app_mod, str(e))))
+    def import_models():
+        """Performs collision resolving and imports all models.
+        When collisions are resolved we can perform imports and print information's, because it is last phase.
+        This function updates imported_objects dictionary.
+        """
+        modules_to_models = CollisionResolvingRunner().run_collision_resolver(get_dict_from_names_to_possible_models())
+        for full_module_path, models in modules_to_models.items():
+            model_labels = []
+            for (model_name, alias) in sorted(models):
+                try:
+                    imported_objects[alias] = import_string("%s.%s" % (full_module_path, model_name))
+                    if model_name == alias:
+                        model_labels.append(model_name)
+                    else:
+                        model_labels.append("%s (as %s)" % (model_name, alias))
+                except ImportError as e:
+                    if options.get("traceback"):
+                        traceback.print_exc()
+                    if not options.get('quiet_load'):
+                        print(style.ERROR(
+                            "Failed to import '%s' from '%s' reason: %s" % (model_name, full_module_path, str(e))))
+            if not options.get('quiet_load'):
+                print(style.SQL_COLTYPE("from %s import %s" % (full_module_path, ", ".join(model_labels))))
+
+    def get_apps_and_models():
+        for app in apps.get_app_configs():
+            if app.models_module:
+                yield app.models_module, app.get_models()
+
+    mongoengine = False
+    try:
+        from mongoengine.base import _document_registry
+        mongoengine = True
+    except ImportError:
+        pass
+
+    # Perform pre-imports before any other imports
+    if SHELL_PLUS_PRE_IMPORTS:
+        if not quiet_load:
+            print(style.SQL_TABLE("# Shell Plus User Imports"))
+        imports = import_items(SHELL_PLUS_PRE_IMPORTS, style, quiet_load=quiet_load)
+        for k, v in six.iteritems(imports):
+            imported_objects[k] = v
+
+    load_models = {}
+
+    if mongoengine and dont_load_any_models:
+        for name, mod in six.iteritems(_document_registry):
+            name = name.split('.')[-1]
+            app_name = mod.__module__.split('.')[-2]
+            if app_name in dont_load or ("%s.%s" % (app_name, name)) in dont_load:
                 continue
 
-        if not quiet_load:
-            print(style.SQL_COLTYPE("from %s import %s" % (app_mod, ", ".join(model_labels))))
+            load_models.setdefault(mod.__module__, [])
+            load_models[mod.__module__].append(name)
+
+    if not dont_load_any_models:
+        for app_mod, app_models in get_apps_and_models():
+            if not app_models:
+                continue
+
+            app_name = app_mod.__name__.split('.')[-2]
+            if app_name in dont_load:
+                continue
+
+            for mod in app_models:
+                if "%s.%s" % (app_name, mod.__name__) in dont_load:
+                    continue
+
+                if mod.__module__:
+                    # Only add the module to the dict if `__module__` is not empty.
+                    load_models.setdefault(mod.__module__, [])
+                    load_models[mod.__module__].append(mod.__name__)
+
+    if not quiet_load:
+        print(style.SQL_TABLE("# Shell Plus Model Imports%s") % (' SKIPPED' if dont_load_any_models else ''))
+
+    import_models()
 
     # Imports often used from Django
     if getattr(settings, 'SHELL_PLUS_DJANGO_IMPORTS', True):
